// Copyleft (cl) 1999, 2000 by Malekith / The Final Crash of Corp. System
// <malekith@topnet.pl> / <http://malekith.topnet.pl>
// This is part of gallery image viewer. See file copying-left for details.
// Created on Aug 10 1999
// $Id: image.cc,v 2.6 1999/12/11 21:54:13 malekith Exp malekith $

#include "main.h"
#include "image.h"
#include <vgagl.h>
#include <fnmatch.h>

#ifdef aalib
#	include <aalib.h>
#endif

// ---------------------------------------------------------------------------
// globals
const byte black_palette[256*3] = { 0 };
// ---------------------------------------------------------------------------
void Bitmap::free()
{
	delete [] buf;
	buf = null;
	w = 0;
	h = 0;
}
// ---------------------------------------------------------------------------
Bitmap::~Bitmap()
{
	free();
}
// ---------------------------------------------------------------------------
void Bitmap::alloc(int x, int y)
{
	free();
	w = x; wbpp = w * bytespp;
	h = y;
	buf = new byte[w * h * bytespp];
}
// ---------------------------------------------------------------------------
void Bitmap::put(Bitmap & target)
{
	memset(target.data(), 0, target.width() * target.height() * bytespp);
	int n = (height()+y <? target.height()) - y;
	int b = ((width()+x <? target.width()) - x) * bytespp;
	byte * d = target.data() + (x + y * target.width()) * bytespp;
	byte * s = data();
	while (n-- > 0) {
		memcpy(d, s, b);
		d += target.line_width();
		s += line_width();
	}
}
// ---------------------------------------------------------------------------
#if 0
static void scale(int w1, int h1, byte * s, int w2, int h2, byte * d,
		  int bpp)
{
	int x = 0, y = 0;
	int n, yy;
	int dy = (h1 << 16) / h2, dx = (w1 << 16) / w2;
	w1 *= bpp;
	for (yy = 0; yy < h2; yy++) {
		byte *ss = s + (y >> 16) * w1;
		y += dy;
		x = 0;
		n = w2;

		switch (bpp) {
		case 1:
			while (n-- > 0) {
				*d++ = ss[x >> 16];
				x += dx;
			}
			break;
		case 2:
			while (n-- > 0) {
				int t = x >> 16;
				t += t;
				*d++ = ss[t++];
				*d++ = ss[t];
				x += dx;
			}
			break;
		case 3:
			while (n-- > 0) {
				int t = x >> 16;
				t = t + t + t;
				*d++ = ss[t++];
				*d++ = ss[t++];
				*d++ = ss[t];
				x += dx;
			}
			break;
		case 4:
			while (n-- > 0) {
				int t = x >> 16;
				t += t;
				t += t;
				*d++ = ss[t++];
				*d++ = ss[t++];
				*d++ = ss[t++];
				*d++ = ss[t];
				x += dx;
			}
			break;
		default:
			abort();
		}
	}
}
void Bitmap::resize_from(const Bitmap & target)
{
	assert(buf);
	assert(target.buf);
	scale(target.w, target.h, target.buf, w, h, buf, bytespp);
	/*
	gl_scalebox(	target.w, target.h, target.buf,
			w, h, buf
			); */
}
#endif
// ---------------------------------------------------------------------------
#ifdef aalib
extern aa_context *getctx();
extern aa_renderparams rendp;
#endif

void ScrBitmap::aaput(int sx, int sy)
{
#	ifdef aalib
	aa_context *ctx = getctx();
	
	assert(ctx);
	assert(x >= 0);
	assert(y >= 0);
	assert(bytespp == 3);
	
	int yy = (y * parent->height() / height()) << 16;
	int dy = ((screen_h * parent->height() / height()) << 16) / aa_imgheight(ctx);
	int dx = (parent->width() << 16) / width();
	byte *t = aa_image(ctx);
	
	memset(t, 0, aa_imgheight(ctx) * aa_imgwidth(ctx));
	
	int n = height() * aa_imgheight(ctx) / screen_h <? aa_imgheight(ctx);
	int b = width() <? aa_imgwidth(ctx);
	byte *d = t + (sx + (sy * aa_imgheight(ctx) / screen_h) * aa_imgwidth(ctx));
	
	byte *s = parent->data() + x * parent->width() / width() * 3;
//LOG("aa(%d,%d) (%d,%d)",sx,sy,x,y);	
	while (n-- > 0) {
		int n1 = b;
		byte *d1 = d, *s1 = s + (yy >> 16) * parent->line_width();
		register int xx = 0;
		
		yy += dy;
		while (n1-- > 0) {
			register byte *p = s1 + (xx >> 16) + (xx >> 16) + (xx >> 16);
			register int k = *p++;
			k += *p + *p++;
			k += *p++;
			*d1++ = k >> 2;
			xx += dx;
		}
		d += aa_imgwidth(ctx);
	}
	aa_render(ctx, &rendp, 0, 0, aa_scrwidth(ctx), aa_scrheight(ctx));
	aa_flush(ctx);
#	else
	abort();
#	endif
}

extern "C"
void x_putboxscaled(int sx, int sy, int sw, int sh,	// on-screen pos/size
		    int bx, int by, int bw, int bh,	// on-bitmap pos/size
		    void *b);

void Bitmap::put()
{
#	ifdef aalib
	if (getctx()) abort();
#	endif	
	gl_enableclipping();
	gl_putbox(-x, -y, width(), height(), data());
}

void ScrBitmap::put()
{
#	ifdef aalib
	if (getctx()) { aaput(0,0); return; }
#	endif	
	x_putboxscaled(
		(-x >? 0), (-y >? 0), width(), height(),
		(x >? 0) * parent->width() / width(), 
		(y >? 0) * parent->height() / height(), 
		parent->width(), parent->height(), parent->data()
	);
}

// ---------------------------------------------------------------------------
Image::Image(const char * p1, const char * p2, const char * p3, 
	     const char * p4, const char * p5, const char * p6)
: magic(null), file_is_temp(false), file_name(null)
{ 
	patterns[0] = p1;
	patterns[1] = p2;
	patterns[2] = p3;
	patterns[3] = p4;
	patterns[4] = p5;
	patterns[5] = p6;
}
// ---------------------------------------------------------------------------
bool Image::match_name(const char * fname)
{
	const char ** p = patterns;
	while (*p) {
		if (fnmatch(*p, fname, FNM_LEADING_DIR | FNM_CASEFOLD) == 0)
			return true;
		p++;
	}
	return false;
}
// ---------------------------------------------------------------------------
bool Image::match_magic(const byte * buffer)
{
	return magic && (strncmp((const char*)buffer, 
				 magic, 
				 strlen(magic)) == 0);
}
// ---------------------------------------------------------------------------
extern "C" void now_doing(int,int);
void Image::read()
{
	now_doing(0,100); // default
	do_read();
	clear_temp();
}
// ---------------------------------------------------------------------------
void Image::clear_temp()
{
	if (file_is_temp) {
		if (strncmp(file_name, "/tmp", 4) != 0)
			throw xRecover(_("won't delete file 'cause it don't "
				"looks like temp")); // should never happen
		remove(file_name);
		file_is_temp = false;
	}
}
// ---------------------------------------------------------------------------
